import { PresentationMode } from 'react-native-theoplayer';
import { DefaultPresentationModeChangeEvent } from '../event/PlayerEvents';
import { fullscreenAPI } from './FullscreenAPI';
import { noOp } from '../../utils/CommonUtils';
export class WebPresentationModeManager {
  _presentationMode = PresentationMode.inline;
  _element = undefined;
  constructor(player, eventForwarder) {
    this._player = player;
    this._eventForwarder = eventForwarder;
    this._player.presentation.addEventListener('presentationmodechange', this.updatePresentationMode);
  }
  get presentationMode() {
    return this._presentationMode;
  }
  set presentationMode(presentationMode) {
    if (presentationMode === this._presentationMode) {
      return;
    }
    this.prepareForPresentationModeChanges();
    if (fullscreenAPI !== undefined) {
      // All other browsers
      if (presentationMode === PresentationMode.fullscreen) {
        const appElement = document.getElementById('app') ?? document.getElementById('root');
        if (appElement !== null) {
          const promise = appElement[fullscreenAPI.requestFullscreen_]();
          if (promise && promise.then) {
            promise.then(noOp, noOp);
          }
        }
      } else if (presentationMode === PresentationMode.pip) {
        this._player.presentation.requestMode('native-picture-in-picture');
      } else {
        if (this._presentationMode === PresentationMode.fullscreen) {
          const promise = document[fullscreenAPI.exitFullscreen_]();
          if (promise && promise.then) {
            promise.then(noOp, noOp);
          }
        }
        if (this._presentationMode === PresentationMode.pip) {
          void document.exitPictureInPicture();
        }
      }
    } else {
      // iOS Safari doesn't properly support fullscreen, use native fullscreen instead
      if (presentationMode === PresentationMode.fullscreen) {
        this._element?.webkitEnterFullscreen?.();
      } else if (presentationMode === PresentationMode.pip) {
        this._player.presentation.requestMode('native-picture-in-picture');
      } else {
        this._element?.webkitSetPresentationMode?.(PresentationMode.inline);
      }
    }
  }
  prepareForPresentationModeChanges() {
    const elements = this._player.element.children;
    for (const element of Array.from(elements)) {
      if (element.tagName === 'VIDEO') {
        const videoElement = element;
        if (videoElement.src !== null && videoElement.src !== '' || videoElement.srcObject !== null) {
          this._element = videoElement;
          break;
        }
      }
    }
    // listen for fullscreen updates on document
    if (fullscreenAPI !== undefined) {
      document.addEventListener(fullscreenAPI.fullscreenchange_, this.updatePresentationMode);
      document.addEventListener(fullscreenAPI.fullscreenerror_, this.updatePresentationMode);
    }
  }
  updatePresentationMode = () => {
    // detect new presentation mode
    let newPresentationMode = PresentationMode.inline;
    if (fullscreenAPI !== undefined && document[fullscreenAPI.fullscreenElement_] !== null) {
      newPresentationMode = PresentationMode.fullscreen;
    } else if (this._player.presentation.currentMode === 'native-picture-in-picture') {
      newPresentationMode = PresentationMode.pip;
    }

    // when changed, notify by dispatching presentationModeChange event
    const previousPresentationMode = this._presentationMode;
    if (newPresentationMode !== previousPresentationMode) {
      this._presentationMode = newPresentationMode;
      this._eventForwarder.dispatchEvent(new DefaultPresentationModeChangeEvent(this._presentationMode, previousPresentationMode));
    }
  };
}
//# sourceMappingURL=WebPresentationModeManager.js.map